@@ -6,7 +6,7 @@
  */
 
 import React from 'react';
-import {unstable_createRoot as createRoot} from 'react-dom';
+import {createRoot} from 'react-dom/client';
 import './index.css';
 import Router from './Router';
 
