@@ -26,6 +26,7 @@ type ConnectOptions = {
   port?: number,
   useHttps?: boolean,
   resolveRNStyle?: ResolveNativeStyle,
+  retryConnectionDelay?: number,
   isAppActive?: () => boolean,
   websocket?: ?WebSocket,
   ...
@@ -60,6 +61,7 @@ export function connectToDevTools(options: ?ConnectOptions) {
     port = 8097,
     websocket,
     resolveRNStyle = null,
+    retryConnectionDelay = 2000,
     isAppActive = () => true,
   } = options || {};
 
@@ -69,7 +71,10 @@ export function connectToDevTools(options: ?ConnectOptions) {
   function scheduleRetry() {
     if (retryTimeoutID === null) {
       // Two seconds because RN had issues with quick retries.
-      retryTimeoutID = setTimeout(() => connectToDevTools(options), 2000);
+      retryTimeoutID = setTimeout(
+        () => connectToDevTools(options),
+        retryConnectionDelay,
+      );
     }
   }
 
