@@ -12,7 +12,7 @@ window.injectStyles = getLinkTags => {
     const linkTags = getLinkTags();
 
     // eslint-disable-next-line no-for-of-loops/no-for-of-loops
-    for (let linkTag of linkTags) {
+    for (const linkTag of linkTags) {
       document.head.appendChild(linkTag);
     }
   }
