@@ -38,7 +38,11 @@ describe('Store (legacy)', () => {
     act(() =>
       ReactDOM.render(<Component count={4} />, document.createElement('div')),
     );
-    expect(store).toMatchSnapshot('1: mount');
+    expect(store).toMatchInlineSnapshot(`
+      [root]
+        ▾ <Component>
+            <div>
+    `);
 
     expect(store.roots).toHaveLength(1);
 
@@ -73,13 +77,53 @@ describe('Store (legacy)', () => {
       const container = document.createElement('div');
 
       act(() => ReactDOM.render(<Grandparent count={4} />, container));
-      expect(store).toMatchSnapshot('1: mount');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▾ <Parent>
+                ▾ <div>
+                  ▾ <Child key="0">
+                      <div>
+                  ▾ <Child key="1">
+                      <div>
+                  ▾ <Child key="2">
+                      <div>
+                  ▾ <Child key="3">
+                      <div>
+              ▾ <Parent>
+                ▾ <div>
+                  ▾ <Child key="0">
+                      <div>
+                  ▾ <Child key="1">
+                      <div>
+                  ▾ <Child key="2">
+                      <div>
+                  ▾ <Child key="3">
+                      <div>
+      `);
 
       act(() => ReactDOM.render(<Grandparent count={2} />, container));
-      expect(store).toMatchSnapshot('2: update');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▾ <Parent>
+                ▾ <div>
+                  ▾ <Child key="0">
+                      <div>
+                  ▾ <Child key="1">
+                      <div>
+              ▾ <Parent>
+                ▾ <div>
+                  ▾ <Child key="0">
+                      <div>
+                  ▾ <Child key="1">
+                      <div>
+      `);
 
       act(() => ReactDOM.unmountComponentAtNode(container));
-      expect(store).toMatchSnapshot('3: unmount');
+      expect(store).toMatchInlineSnapshot(``);
     });
 
     it('should support mount and update operations for multiple roots', () => {
@@ -99,19 +143,65 @@ describe('Store (legacy)', () => {
         ReactDOM.render(<Parent key="A" count={3} />, containerA);
         ReactDOM.render(<Parent key="B" count={2} />, containerB);
       });
-      expect(store).toMatchSnapshot('1: mount');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Parent key="A">
+            ▾ <div>
+              ▾ <Child key="0">
+                  <div>
+              ▾ <Child key="1">
+                  <div>
+              ▾ <Child key="2">
+                  <div>
+        [root]
+          ▾ <Parent key="B">
+            ▾ <div>
+              ▾ <Child key="0">
+                  <div>
+              ▾ <Child key="1">
+                  <div>
+      `);
 
       act(() => {
         ReactDOM.render(<Parent key="A" count={4} />, containerA);
         ReactDOM.render(<Parent key="B" count={1} />, containerB);
       });
-      expect(store).toMatchSnapshot('2: update');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Parent key="A">
+            ▾ <div>
+              ▾ <Child key="0">
+                  <div>
+              ▾ <Child key="1">
+                  <div>
+              ▾ <Child key="2">
+                  <div>
+              ▾ <Child key="3">
+                  <div>
+        [root]
+          ▾ <Parent key="B">
+            ▾ <div>
+              ▾ <Child key="0">
+                  <div>
+      `);
 
       act(() => ReactDOM.unmountComponentAtNode(containerB));
-      expect(store).toMatchSnapshot('3: unmount B');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Parent key="A">
+            ▾ <div>
+              ▾ <Child key="0">
+                  <div>
+              ▾ <Child key="1">
+                  <div>
+              ▾ <Child key="2">
+                  <div>
+              ▾ <Child key="3">
+                  <div>
+      `);
 
       act(() => ReactDOM.unmountComponentAtNode(containerA));
-      expect(store).toMatchSnapshot('4: unmount A');
+      expect(store).toMatchInlineSnapshot(``);
     });
 
     it('should not filter DOM nodes from the store tree', () => {
@@ -139,15 +229,43 @@ describe('Store (legacy)', () => {
       act(() =>
         ReactDOM.render(<Grandparent count={4} flip={false} />, container),
       );
-      expect(store).toMatchSnapshot('1: mount');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▾ <div>
+                ▾ <Parent>
+                  ▾ <div>
+                    ▾ <Child>
+                        <div>
+              ▾ <Parent>
+                ▾ <div>
+                  ▾ <Child>
+                      <div>
+                <Nothing>
+      `);
 
       act(() =>
         ReactDOM.render(<Grandparent count={4} flip={true} />, container),
       );
-      expect(store).toMatchSnapshot('2: update');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▾ <div>
+                ▾ <Parent>
+                  ▾ <div>
+                    ▾ <Child>
+                        <div>
+              ▾ <Parent>
+                ▾ <div>
+                  ▾ <Child>
+                      <div>
+                <Nothing>
+      `);
 
       act(() => ReactDOM.unmountComponentAtNode(container));
-      expect(store).toMatchSnapshot('5: unmount');
+      expect(store).toMatchInlineSnapshot(``);
     });
 
     it('should support collapsing parts of the tree', () => {
@@ -172,26 +290,84 @@ describe('Store (legacy)', () => {
           document.createElement('div'),
         ),
       );
-      expect(store).toMatchSnapshot('1: mount');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▾ <Parent>
+                ▾ <div>
+                  ▾ <Child key="0">
+                      <div>
+                  ▾ <Child key="1">
+                      <div>
+              ▾ <Parent>
+                ▾ <div>
+                  ▾ <Child key="0">
+                      <div>
+                  ▾ <Child key="1">
+                      <div>
+      `);
 
       const grandparentID = store.getElementIDAtIndex(0);
       const parentOneID = store.getElementIDAtIndex(2);
       const parentTwoID = store.getElementIDAtIndex(8);
 
       act(() => store.toggleIsCollapsed(parentOneID, true));
-      expect(store).toMatchSnapshot('2: collapse first Parent');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▸ <Parent>
+              ▾ <Parent>
+                ▾ <div>
+                  ▾ <Child key="0">
+                      <div>
+                  ▾ <Child key="1">
+                      <div>
+      `);
 
       act(() => store.toggleIsCollapsed(parentTwoID, true));
-      expect(store).toMatchSnapshot('3: collapse second Parent');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▸ <Parent>
+              ▸ <Parent>
+      `);
 
       act(() => store.toggleIsCollapsed(parentOneID, false));
-      expect(store).toMatchSnapshot('4: expand first Parent');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▾ <Parent>
+                ▾ <div>
+                  ▾ <Child key="0">
+                      <div>
+                  ▾ <Child key="1">
+                      <div>
+              ▸ <Parent>
+      `);
 
       act(() => store.toggleIsCollapsed(grandparentID, true));
-      expect(store).toMatchSnapshot('5: collapse Grandparent');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Grandparent>
+      `);
 
       act(() => store.toggleIsCollapsed(grandparentID, false));
-      expect(store).toMatchSnapshot('6: expand Grandparent');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▾ <Parent>
+                ▾ <div>
+                  ▾ <Child key="0">
+                      <div>
+                  ▾ <Child key="1">
+                      <div>
+              ▸ <Parent>
+      `);
     });
 
     it('should support adding and removing children', () => {
@@ -208,7 +384,13 @@ describe('Store (legacy)', () => {
           container,
         ),
       );
-      expect(store).toMatchSnapshot('1: mount');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Root>
+            ▾ <div>
+              ▾ <Component key="a">
+                  <div>
+      `);
 
       act(() =>
         ReactDOM.render(
@@ -219,7 +401,15 @@ describe('Store (legacy)', () => {
           container,
         ),
       );
-      expect(store).toMatchSnapshot('2: add child');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Root>
+            ▾ <div>
+              ▾ <Component key="a">
+                  <div>
+              ▾ <Component key="b">
+                  <div>
+      `);
 
       act(() =>
         ReactDOM.render(
@@ -229,7 +419,13 @@ describe('Store (legacy)', () => {
           container,
         ),
       );
-      expect(store).toMatchSnapshot('3: remove child');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Root>
+            ▾ <div>
+              ▾ <Component key="b">
+                  <div>
+      `);
     });
 
     it('should support reordering of children', () => {
@@ -246,16 +442,61 @@ describe('Store (legacy)', () => {
       const container = document.createElement('div');
 
       act(() => ReactDOM.render(<Root>{[foo, bar]}</Root>, container));
-      expect(store).toMatchSnapshot('1: mount');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Root>
+            ▾ <div>
+              ▾ <Foo key="foo">
+                ▾ <div>
+                  ▾ <Component key="0">
+                      <div>
+              ▾ <Bar key="bar">
+                ▾ <div>
+                  ▾ <Component key="0">
+                      <div>
+                  ▾ <Component key="1">
+                      <div>
+      `);
 
       act(() => ReactDOM.render(<Root>{[bar, foo]}</Root>, container));
-      expect(store).toMatchSnapshot('2: reorder children');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Root>
+            ▾ <div>
+              ▾ <Bar key="bar">
+                ▾ <div>
+                  ▾ <Component key="0">
+                      <div>
+                  ▾ <Component key="1">
+                      <div>
+              ▾ <Foo key="foo">
+                ▾ <div>
+                  ▾ <Component key="0">
+                      <div>
+      `);
 
       act(() => store.toggleIsCollapsed(store.getElementIDAtIndex(0), true));
-      expect(store).toMatchSnapshot('3: collapse root');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Root>
+      `);
 
       act(() => store.toggleIsCollapsed(store.getElementIDAtIndex(0), false));
-      expect(store).toMatchSnapshot('4: expand root');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Root>
+            ▾ <div>
+              ▾ <Bar key="bar">
+                ▾ <div>
+                  ▾ <Component key="0">
+                      <div>
+                  ▾ <Component key="1">
+                      <div>
+              ▾ <Foo key="foo">
+                ▾ <div>
+                  ▾ <Component key="0">
+                      <div>
+      `);
     });
   });
 
@@ -285,7 +526,10 @@ describe('Store (legacy)', () => {
           container,
         ),
       );
-      expect(store).toMatchSnapshot('1: mount');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <div>
+      `);
 
       act(() =>
         ReactDOM.render(
@@ -296,10 +540,13 @@ describe('Store (legacy)', () => {
           container,
         ),
       );
-      expect(store).toMatchSnapshot('2: update');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <div>
+      `);
 
       act(() => ReactDOM.unmountComponentAtNode(container));
-      expect(store).toMatchSnapshot('3: unmount');
+      expect(store).toMatchInlineSnapshot(``);
     });
 
     it('should support mount and update operations for multiple roots', () => {
@@ -319,19 +566,32 @@ describe('Store (legacy)', () => {
         ReactDOM.render(<Parent key="A" count={3} />, containerA);
         ReactDOM.render(<Parent key="B" count={2} />, containerB);
       });
-      expect(store).toMatchSnapshot('1: mount');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Parent key="A">
+        [root]
+          ▸ <Parent key="B">
+      `);
 
       act(() => {
         ReactDOM.render(<Parent key="A" count={4} />, containerA);
         ReactDOM.render(<Parent key="B" count={1} />, containerB);
       });
-      expect(store).toMatchSnapshot('2: update');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Parent key="A">
+        [root]
+          ▸ <Parent key="B">
+      `);
 
       act(() => ReactDOM.unmountComponentAtNode(containerB));
-      expect(store).toMatchSnapshot('3: unmount B');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Parent key="A">
+      `);
 
       act(() => ReactDOM.unmountComponentAtNode(containerA));
-      expect(store).toMatchSnapshot('4: unmount A');
+      expect(store).toMatchInlineSnapshot(``);
     });
 
     it('should not filter DOM nodes from the store tree', () => {
@@ -359,21 +619,42 @@ describe('Store (legacy)', () => {
       act(() =>
         ReactDOM.render(<Grandparent count={4} flip={false} />, container),
       );
-      expect(store).toMatchSnapshot('1: mount');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Grandparent>
+      `);
 
       act(() => store.toggleIsCollapsed(store.getElementIDAtIndex(0), false));
-      expect(store).toMatchSnapshot('2: expand Grandparent');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▸ <div>
+      `);
 
       act(() => store.toggleIsCollapsed(store.getElementIDAtIndex(1), false));
-      expect(store).toMatchSnapshot('3: expand div');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▸ <div>
+              ▸ <Parent>
+                <Nothing>
+      `);
 
       act(() =>
         ReactDOM.render(<Grandparent count={4} flip={true} />, container),
       );
-      expect(store).toMatchSnapshot('4: final update');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▸ <div>
+              ▸ <Parent>
+                <Nothing>
+      `);
 
       act(() => ReactDOM.unmountComponentAtNode(container));
-      expect(store).toMatchSnapshot('5: unmount');
+      expect(store).toMatchInlineSnapshot(``);
     });
 
     it('should support expanding parts of the tree', () => {
@@ -398,34 +679,78 @@ describe('Store (legacy)', () => {
           document.createElement('div'),
         ),
       );
-      expect(store).toMatchSnapshot('1: mount');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Grandparent>
+      `);
 
       const grandparentID = store.getElementIDAtIndex(0);
 
       act(() => store.toggleIsCollapsed(grandparentID, false));
-      expect(store).toMatchSnapshot('2: expand Grandparent');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▸ <div>
+      `);
 
       const parentDivID = store.getElementIDAtIndex(1);
       act(() => store.toggleIsCollapsed(parentDivID, false));
-      expect(store).toMatchSnapshot('3: expand parent div');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▸ <Parent>
+              ▸ <Parent>
+      `);
 
       const parentOneID = store.getElementIDAtIndex(2);
       const parentTwoID = store.getElementIDAtIndex(3);
 
       act(() => store.toggleIsCollapsed(parentOneID, false));
-      expect(store).toMatchSnapshot('4: expand first Parent');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▾ <Parent>
+                ▸ <div>
+              ▸ <Parent>
+      `);
 
       act(() => store.toggleIsCollapsed(parentTwoID, false));
-      expect(store).toMatchSnapshot('5: expand second Parent');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▾ <Parent>
+                ▸ <div>
+              ▾ <Parent>
+                ▸ <div>
+      `);
 
       act(() => store.toggleIsCollapsed(parentOneID, true));
-      expect(store).toMatchSnapshot('6: collapse first Parent');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▸ <Parent>
+              ▾ <Parent>
+                ▸ <div>
+      `);
 
       act(() => store.toggleIsCollapsed(parentTwoID, true));
-      expect(store).toMatchSnapshot('7: collapse second Parent');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Grandparent>
+            ▾ <div>
+              ▸ <Parent>
+              ▸ <Parent>
+      `);
 
       act(() => store.toggleIsCollapsed(grandparentID, true));
-      expect(store).toMatchSnapshot('8: collapse Grandparent');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Grandparent>
+      `);
     });
 
     it('should support expanding deep parts of the tree', () => {
@@ -450,28 +775,62 @@ describe('Store (legacy)', () => {
           document.createElement('div'),
         ),
       );
-      expect(store).toMatchSnapshot('1: mount');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Wrapper>
+      `);
 
       const deepestedNodeID = global.agent.getIDForNode(ref);
 
       act(() => store.toggleIsCollapsed(deepestedNodeID, false));
-      expect(store).toMatchSnapshot('2: expand deepest node');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Wrapper>
+            ▾ <Nested>
+              ▾ <Nested>
+                ▾ <Nested>
+                  ▾ <Nested>
+                      <div>
+      `);
 
       const rootID = store.getElementIDAtIndex(0);
 
       act(() => store.toggleIsCollapsed(rootID, true));
-      expect(store).toMatchSnapshot('3: collapse root');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Wrapper>
+      `);
 
       act(() => store.toggleIsCollapsed(rootID, false));
-      expect(store).toMatchSnapshot('4: expand root');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Wrapper>
+            ▾ <Nested>
+              ▾ <Nested>
+                ▾ <Nested>
+                  ▾ <Nested>
+                      <div>
+      `);
 
       const id = store.getElementIDAtIndex(1);
 
       act(() => store.toggleIsCollapsed(id, true));
-      expect(store).toMatchSnapshot('5: collapse middle node');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Wrapper>
+            ▸ <Nested>
+      `);
 
       act(() => store.toggleIsCollapsed(id, false));
-      expect(store).toMatchSnapshot('6: expand middle node');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Wrapper>
+            ▾ <Nested>
+              ▾ <Nested>
+                ▾ <Nested>
+                  ▾ <Nested>
+                      <div>
+      `);
     });
 
     it('should support reordering of children', () => {
@@ -488,25 +847,63 @@ describe('Store (legacy)', () => {
       const container = document.createElement('div');
 
       act(() => ReactDOM.render(<Root>{[foo, bar]}</Root>, container));
-      expect(store).toMatchSnapshot('1: mount');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Root>
+      `);
 
       act(() => ReactDOM.render(<Root>{[bar, foo]}</Root>, container));
-      expect(store).toMatchSnapshot('2: reorder children');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Root>
+      `);
 
       act(() => store.toggleIsCollapsed(store.getElementIDAtIndex(0), false));
-      expect(store).toMatchSnapshot('3: expand root');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Root>
+            ▸ <div>
+      `);
 
       act(() => store.toggleIsCollapsed(store.getElementIDAtIndex(1), false));
-      expect(store).toMatchSnapshot('4: expand div');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Root>
+            ▾ <div>
+              ▸ <Bar key="bar">
+              ▸ <Foo key="foo">
+      `);
 
       act(() => {
         store.toggleIsCollapsed(store.getElementIDAtIndex(3), false);
         store.toggleIsCollapsed(store.getElementIDAtIndex(2), false);
       });
-      expect(store).toMatchSnapshot('4: expand leaves');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▾ <Root>
+            ▾ <div>
+              ▾ <Bar key="bar">
+                ▸ <div>
+              ▾ <Foo key="foo">
+                ▸ <div>
+      `);
 
       act(() => store.toggleIsCollapsed(store.getElementIDAtIndex(0), true));
-      expect(store).toMatchSnapshot('5: collapse root');
+      expect(store).toMatchInlineSnapshot(`
+        [root]
+          ▸ <Root>
+      `);
+    });
+  });
+
+  describe('StrictMode compliance', () => {
+    it('should mark all elements as strict mode compliant', () => {
+      const App = () => null;
+
+      const container = document.createElement('div');
+      act(() => ReactDOM.render(<App />, container));
+
+      expect(store.getElementAtIndex(0).isStrictModeNonCompliant).toBe(false);
     });
   });
 });
