@@ -14,9 +14,9 @@ import type Store from 'react-devtools-shared/src/devtools/store';
 
 describe('OwnersListContext', () => {
   let React;
-  let ReactDOM;
   let TestRenderer: ReactTestRenderer;
   let bridge: FrontendBridge;
+  let legacyRender;
   let store: Store;
   let utils;
 
@@ -30,12 +30,13 @@ describe('OwnersListContext', () => {
     utils = require('./utils');
     utils.beforeEachProfiling();
 
+    legacyRender = utils.legacyRender;
+
     bridge = global.bridge;
     store = global.store;
     store.collapseNodesByDefault = false;
 
     React = require('react');
-    ReactDOM = require('react-dom');
     TestRenderer = utils.requireTestRenderer();
 
     BridgeContext = require('react-devtools-shared/src/devtools/views/context')
@@ -60,7 +61,32 @@ describe('OwnersListContext', () => {
     </BridgeContext.Provider>
   );
 
-  it('should fetch the owners list for the selected element', async done => {
+  async function getOwnersListForOwner(owner) {
+    let ownerDisplayNames = null;
+
+    function Suspender() {
+      const read = React.useContext(OwnersListContext);
+      const owners = read(owner.id);
+      ownerDisplayNames = owners.map(({displayName}) => displayName);
+      return null;
+    }
+
+    await utils.actAsync(() =>
+      TestRenderer.create(
+        <Contexts defaultOwnerID={owner.id}>
+          <React.Suspense fallback={null}>
+            <Suspender owner={owner} />
+          </React.Suspense>
+        </Contexts>,
+      ),
+    );
+
+    expect(ownerDisplayNames).not.toBeNull();
+
+    return ownerDisplayNames;
+  }
+
+  it('should fetch the owners list for the selected element', async () => {
     const Grandparent = () => <Parent />;
     const Parent = () => {
       return (
@@ -73,53 +99,37 @@ describe('OwnersListContext', () => {
     const Child = () => null;
 
     utils.act(() =>
-      ReactDOM.render(<Grandparent />, document.createElement('div')),
+      legacyRender(<Grandparent />, document.createElement('div')),
     );
 
-    expect(store).toMatchSnapshot('mount');
+    expect(store).toMatchInlineSnapshot(`
+      [root]
+        ▾ <Grandparent>
+          ▾ <Parent>
+              <Child>
+              <Child>
+    `);
 
     const parent = ((store.getElementAtIndex(1): any): Element);
     const firstChild = ((store.getElementAtIndex(2): any): Element);
 
-    let didFinish = false;
+    expect(await getOwnersListForOwner(parent)).toMatchInlineSnapshot(`
+      Array [
+        "Grandparent",
+        "Parent",
+      ]
+    `);
 
-    function Suspender({owner}) {
-      const read = React.useContext(OwnersListContext);
-      const owners = read(owner.id);
-      expect(owners).toMatchSnapshot(
-        `owners for "${(owner && owner.displayName) || ''}"`,
-      );
-      didFinish = true;
-      return null;
-    }
-
-    await utils.actAsync(() =>
-      TestRenderer.create(
-        <Contexts defaultOwnerID={parent.id}>
-          <React.Suspense fallback={null}>
-            <Suspender owner={parent} />
-          </React.Suspense>
-        </Contexts>,
-      ),
-    );
-    expect(didFinish).toBe(true);
-
-    didFinish = false;
-    await utils.actAsync(() =>
-      TestRenderer.create(
-        <Contexts defaultOwnerID={firstChild.id}>
-          <React.Suspense fallback={null}>
-            <Suspender owner={firstChild} />
-          </React.Suspense>
-        </Contexts>,
-      ),
-    );
-    expect(didFinish).toBe(true);
-
-    done();
+    expect(await getOwnersListForOwner(firstChild)).toMatchInlineSnapshot(`
+      Array [
+        "Grandparent",
+        "Parent",
+        "Child",
+      ]
+    `);
   });
 
-  it('should fetch the owners list for the selected element that includes filtered components', async done => {
+  it('should fetch the owners list for the selected element that includes filtered components', async () => {
     store.componentFilters = [utils.createDisplayNameFilter('^Parent$')];
 
     const Grandparent = () => <Parent />;
@@ -134,80 +144,52 @@ describe('OwnersListContext', () => {
     const Child = () => null;
 
     utils.act(() =>
-      ReactDOM.render(<Grandparent />, document.createElement('div')),
+      legacyRender(<Grandparent />, document.createElement('div')),
     );
 
-    expect(store).toMatchSnapshot('mount');
+    expect(store).toMatchInlineSnapshot(`
+      [root]
+        ▾ <Grandparent>
+            <Child>
+            <Child>
+    `);
 
     const firstChild = ((store.getElementAtIndex(1): any): Element);
 
-    let didFinish = false;
-
-    function Suspender({owner}) {
-      const read = React.useContext(OwnersListContext);
-      const owners = read(owner.id);
-      expect(owners).toMatchSnapshot(
-        `owners for "${(owner && owner.displayName) || ''}"`,
-      );
-      didFinish = true;
-      return null;
-    }
-
-    await utils.actAsync(() =>
-      TestRenderer.create(
-        <Contexts defaultOwnerID={firstChild.id}>
-          <React.Suspense fallback={null}>
-            <Suspender owner={firstChild} />
-          </React.Suspense>
-        </Contexts>,
-      ),
-    );
-    expect(didFinish).toBe(true);
-
-    done();
+    expect(await getOwnersListForOwner(firstChild)).toMatchInlineSnapshot(`
+      Array [
+        "Grandparent",
+        "Parent",
+        "Child",
+      ]
+    `);
   });
 
-  it('should include the current element even if there are no other owners', async done => {
+  it('should include the current element even if there are no other owners', async () => {
     store.componentFilters = [utils.createDisplayNameFilter('^Parent$')];
 
     const Grandparent = () => <Parent />;
     const Parent = () => null;
 
     utils.act(() =>
-      ReactDOM.render(<Grandparent />, document.createElement('div')),
+      legacyRender(<Grandparent />, document.createElement('div')),
     );
 
-    expect(store).toMatchSnapshot('mount');
+    expect(store).toMatchInlineSnapshot(`
+      [root]
+          <Grandparent>
+    `);
 
     const grandparent = ((store.getElementAtIndex(0): any): Element);
 
-    let didFinish = false;
-
-    function Suspender({owner}) {
-      const read = React.useContext(OwnersListContext);
-      const owners = read(owner.id);
-      expect(owners).toMatchSnapshot(
-        `owners for "${(owner && owner.displayName) || ''}"`,
-      );
-      didFinish = true;
-      return null;
-    }
-
-    await utils.actAsync(() =>
-      TestRenderer.create(
-        <Contexts defaultOwnerID={grandparent.id}>
-          <React.Suspense fallback={null}>
-            <Suspender owner={grandparent} />
-          </React.Suspense>
-        </Contexts>,
-      ),
-    );
-    expect(didFinish).toBe(true);
-
-    done();
+    expect(await getOwnersListForOwner(grandparent)).toMatchInlineSnapshot(`
+      Array [
+        "Grandparent",
+      ]
+    `);
   });
 
-  it('should include all owners for a component wrapped in react memo', async done => {
+  it('should include all owners for a component wrapped in react memo', async () => {
     const InnerComponent = (props, ref) => <div ref={ref} />;
     const ForwardRef = React.forwardRef(InnerComponent);
     const Memo = React.memo(ForwardRef);
@@ -217,32 +199,24 @@ describe('OwnersListContext', () => {
     };
 
     utils.act(() =>
-      ReactDOM.render(<Grandparent />, document.createElement('div')),
+      legacyRender(<Grandparent />, document.createElement('div')),
     );
 
-    let didFinish = false;
-    function Suspender({owner}) {
-      const read = React.useContext(OwnersListContext);
-      const owners = read(owner.id);
-      didFinish = true;
-      expect(owners.length).toBe(3);
-      expect(owners).toMatchSnapshot(
-        `owners for "${(owner && owner.displayName) || ''}"`,
-      );
-      return null;
-    }
+    expect(store).toMatchInlineSnapshot(`
+      [root]
+        ▾ <Grandparent>
+          ▾ <InnerComponent> [Memo]
+              <InnerComponent> [ForwardRef]
+    `);
 
     const wrapped = ((store.getElementAtIndex(2): any): Element);
-    await utils.actAsync(() =>
-      TestRenderer.create(
-        <Contexts defaultOwnerID={wrapped.id}>
-          <React.Suspense fallback={null}>
-            <Suspender owner={wrapped} />
-          </React.Suspense>
-        </Contexts>,
-      ),
-    );
-    expect(didFinish).toBe(true);
-    done();
+
+    expect(await getOwnersListForOwner(wrapped)).toMatchInlineSnapshot(`
+      Array [
+        "Grandparent",
+        "InnerComponent",
+        "InnerComponent",
+      ]
+    `);
   });
 });
