@@ -8,10 +8,6 @@
  */
 
 import * as React from 'react';
-import {
-  ElementTypeForwardRef,
-  ElementTypeMemo,
-} from 'react-devtools-shared/src/types';
 import styles from './HocBadges.css';
 
 import type {Element} from './types';
@@ -21,22 +17,14 @@ type Props = {|
 |};
 
 export default function HocBadges({element}: Props) {
-  const {hocDisplayNames, type} = ((element: any): Element);
+  const {hocDisplayNames} = ((element: any): Element);
 
-  let typeBadge = null;
-  if (type === ElementTypeMemo) {
-    typeBadge = 'Memo';
-  } else if (type === ElementTypeForwardRef) {
-    typeBadge = 'ForwardRef';
-  }
-
-  if (hocDisplayNames === null && typeBadge === null) {
+  if (hocDisplayNames === null) {
     return null;
   }
 
   return (
     <div className={styles.HocBadges}>
-      {typeBadge !== null && <div className={styles.Badge}>{typeBadge}</div>}
       {hocDisplayNames !== null &&
         hocDisplayNames.map(hocDisplayName => (
           <div key={hocDisplayName} className={styles.Badge}>
