@@ -21,13 +21,17 @@ import {useModalDismissSignal} from './hooks';
 
 import styles from './ModalDialog.css';
 
+type ID = any;
+
 type DIALOG_ACTION_HIDE = {|
   type: 'HIDE',
+  id: ID,
 |};
 type DIALOG_ACTION_SHOW = {|
   type: 'SHOW',
   canBeDismissed?: boolean,
   content: React$Node,
+  id: ID,
   title?: React$Node | null,
 |};
 
@@ -35,13 +39,17 @@ type Action = DIALOG_ACTION_HIDE | DIALOG_ACTION_SHOW;
 
 type Dispatch = (action: Action) => void;
 
-type State = {|
+type Dialog = {|
   canBeDismissed: boolean,
   content: React$Node | null,
-  isVisible: boolean,
+  id: ID,
   title: React$Node | null,
 |};
 
+type State = {|
+  dialogs: Array<Dialog>,
+|};
+
 type ModalDialogContextType = {|
   ...State,
   dispatch: Dispatch,
@@ -56,17 +64,19 @@ function dialogReducer(state, action) {
   switch (action.type) {
     case 'HIDE':
       return {
-        canBeDismissed: true,
-        content: null,
-        isVisible: false,
-        title: null,
+        dialogs: state.dialogs.filter(dialog => dialog.id !== action.id),
       };
     case 'SHOW':
       return {
-        canBeDismissed: action.canBeDismissed !== false,
-        content: action.content,
-        isVisible: true,
-        title: action.title || null,
+        dialogs: [
+          ...state.dialogs,
+          {
+            canBeDismissed: action.canBeDismissed !== false,
+            content: action.content,
+            id: action.id,
+            title: action.title || null,
+          },
+        ],
       };
     default:
       throw new Error(`Invalid action "${action.type}"`);
@@ -79,18 +89,12 @@ type Props = {|
 
 function ModalDialogContextController({children}: Props) {
   const [state, dispatch] = useReducer<State, State, Action>(dialogReducer, {
-    canBeDismissed: true,
-    content: null,
-    isVisible: false,
-    title: null,
+    dialogs: [],
   });
 
   const value = useMemo<ModalDialogContextType>(
     () => ({
-      canBeDismissed: state.canBeDismissed,
-      content: state.content,
-      isVisible: state.isVisible,
-      title: state.title,
+      dialogs: state.dialogs,
       dispatch,
     }),
     [state, dispatch],
@@ -104,17 +108,44 @@ function ModalDialogContextController({children}: Props) {
 }
 
 function ModalDialog(_: {||}) {
-  const {isVisible} = useContext(ModalDialogContext);
-  return isVisible ? <ModalDialogImpl /> : null;
+  const {dialogs, dispatch} = useContext(ModalDialogContext);
+
+  if (dialogs.length === 0) {
+    return null;
+  }
+
+  return (
+    <div className={styles.Background}>
+      {dialogs.map(dialog => (
+        <ModalDialogImpl
+          key={dialog.id}
+          canBeDismissed={dialog.canBeDismissed}
+          content={dialog.content}
+          dispatch={dispatch}
+          id={dialog.id}
+          title={dialog.title}
+        />
+      ))}
+    </div>
+  );
 }
 
-function ModalDialogImpl(_: {||}) {
-  const {canBeDismissed, content, dispatch, title} = useContext(
-    ModalDialogContext,
-  );
+function ModalDialogImpl({
+  canBeDismissed,
+  content,
+  dispatch,
+  id,
+  title,
+}: {|
+  canBeDismissed: boolean,
+  content: React$Node | null,
+  dispatch: Dispatch,
+  id: ID,
+  title: React$Node | null,
+|}) {
   const dismissModal = useCallback(() => {
     if (canBeDismissed) {
-      dispatch({type: 'HIDE'});
+      dispatch({type: 'HIDE', id});
     }
   }, [canBeDismissed, dispatch]);
   const dialogRef = useRef<HTMLDivElement | null>(null);
@@ -135,24 +166,19 @@ function ModalDialogImpl(_: {||}) {
   };
 
   return (
-    <div className={styles.Background} onClick={dismissModal}>
-      <div
-        ref={dialogRef}
-        className={styles.Dialog}
-        onClick={handleDialogClick}>
-        {title !== null && <div className={styles.Title}>{title}</div>}
-        {content}
-        {canBeDismissed && (
-          <div className={styles.Buttons}>
-            <Button
-              autoFocus={true}
-              className={styles.Button}
-              onClick={dismissModal}>
-              Okay
-            </Button>
-          </div>
-        )}
-      </div>
+    <div ref={dialogRef} className={styles.Dialog} onClick={handleDialogClick}>
+      {title !== null && <div className={styles.Title}>{title}</div>}
+      {content}
+      {canBeDismissed && (
+        <div className={styles.Buttons}>
+          <Button
+            autoFocus={true}
+            className={styles.Button}
+            onClick={dismissModal}>
+            Okay
+          </Button>
+        </div>
+      )}
     </div>
   );
 }
