@@ -11,8 +11,11 @@ import {
   __DEBUG__,
   TREE_OPERATION_ADD,
   TREE_OPERATION_REMOVE,
+  TREE_OPERATION_REMOVE_ROOT,
   TREE_OPERATION_REORDER_CHILDREN,
+  TREE_OPERATION_SET_SUBTREE_MODE,
   TREE_OPERATION_UPDATE_TREE_BASE_DURATION,
+  TREE_OPERATION_UPDATE_ERRORS_OR_WARNINGS,
 } from 'react-devtools-shared/src/constants';
 import {utfDecodeString} from 'react-devtools-shared/src/utils';
 import {ElementTypeRoot} from 'react-devtools-shared/src/types';
@@ -54,7 +57,6 @@ export function getCommitTree({
   const commitTrees = ((rootToCommitTreeMap.get(
     rootID,
   ): any): Array<CommitTree>);
-
   if (commitIndex < commitTrees.length) {
     return commitTrees[commitIndex];
   }
@@ -70,52 +72,46 @@ export function getCommitTree({
   }
 
   const {operations} = dataForRoot;
+  if (operations.length <= commitIndex) {
+    throw Error(
+      `getCommitTree(): Invalid commit "${commitIndex}" for root "${rootID}". There are only "${operations.length}" commits.`,
+    );
+  }
 
-  // Commits are generated sequentially and cached.
-  // If this is the very first commit, start with the cached snapshot and apply the first mutation.
-  // Otherwise load (or generate) the previous commit and append a mutation to it.
-  if (commitIndex === 0) {
-    const nodes = new Map();
+  let commitTree: CommitTree = ((null: any): CommitTree);
+  for (let index = commitTrees.length; index <= commitIndex; index++) {
+    // Commits are generated sequentially and cached.
+    // If this is the very first commit, start with the cached snapshot and apply the first mutation.
+    // Otherwise load (or generate) the previous commit and append a mutation to it.
+    if (index === 0) {
+      const nodes = new Map();
 
-    // Construct the initial tree.
-    recursivelyInitializeTree(rootID, 0, nodes, dataForRoot);
+      // Construct the initial tree.
+      recursivelyInitializeTree(rootID, 0, nodes, dataForRoot);
 
-    // Mutate the tree
-    if (operations != null && commitIndex < operations.length) {
-      const commitTree = updateTree({nodes, rootID}, operations[commitIndex]);
+      // Mutate the tree
+      if (operations != null && index < operations.length) {
+        commitTree = updateTree({nodes, rootID}, operations[index]);
+
+        if (__DEBUG__) {
+          __printTree(commitTree);
+        }
+
+        commitTrees.push(commitTree);
+      }
+    } else {
+      const previousCommitTree = commitTrees[index - 1];
+      commitTree = updateTree(previousCommitTree, operations[index]);
 
       if (__DEBUG__) {
         __printTree(commitTree);
       }
 
       commitTrees.push(commitTree);
-      return commitTree;
-    }
-  } else {
-    const previousCommitTree = getCommitTree({
-      commitIndex: commitIndex - 1,
-      profilerStore,
-      rootID,
-    });
-
-    if (operations != null && commitIndex < operations.length) {
-      const commitTree = updateTree(
-        previousCommitTree,
-        operations[commitIndex],
-      );
-
-      if (__DEBUG__) {
-        __printTree(commitTree);
-      }
-
-      commitTrees.push(commitTree);
-      return commitTree;
     }
   }
 
-  throw Error(
-    `getCommitTree(): Unable to reconstruct tree for root "${rootID}" and commit ${commitIndex}`,
-  );
+  return commitTree;
 }
 
 function recursivelyInitializeTree(
@@ -184,7 +180,7 @@ function updateTree(
     const operation = operations[i];
 
     switch (operation) {
-      case TREE_OPERATION_ADD:
+      case TREE_OPERATION_ADD: {
         id = ((operations[i + 1]: any): number);
         const type = ((operations[i + 2]: any): ElementType);
 
@@ -192,14 +188,14 @@ function updateTree(
 
         if (nodes.has(id)) {
           throw new Error(
-            'Commit tree already contains fiber ' +
-              id +
-              '. This is a bug in React DevTools.',
+            `Commit tree already contains fiber "${id}". This is a bug in React DevTools.`,
           );
         }
 
         if (type === ElementTypeRoot) {
-          i++; // supportsProfiling flag
+          i++; // isStrictModeCompliant
+          i++; // Profiling flag
+          i++; // supportsStrictMode flag
           i++; // hasOwnerMetadata flag
 
           if (__DEBUG__) {
@@ -257,6 +253,7 @@ function updateTree(
         }
 
         break;
+      }
       case TREE_OPERATION_REMOVE: {
         const removeLength = ((operations[i + 1]: any): number);
         i += 2;
@@ -267,9 +264,7 @@ function updateTree(
 
           if (!nodes.has(id)) {
             throw new Error(
-              'Commit tree does not contain fiber ' +
-                id +
-                '. This is a bug in React DevTools.',
+              `Commit tree does not contain fiber "${id}". This is a bug in React DevTools.`,
             );
           }
 
@@ -294,6 +289,9 @@ function updateTree(
         }
         break;
       }
+      case TREE_OPERATION_REMOVE_ROOT: {
+        throw Error('Operation REMOVE_ROOT is not supported while profiling.');
+      }
       case TREE_OPERATION_REORDER_CHILDREN: {
         id = ((operations[i + 1]: any): number);
         const numChildren = ((operations[i + 2]: any): number);
@@ -313,6 +311,17 @@ function updateTree(
 
         break;
       }
+      case TREE_OPERATION_SET_SUBTREE_MODE: {
+        id = operations[i + 1];
+        const mode = operations[i + 1];
+
+        i += 3;
+
+        if (__DEBUG__) {
+          debug('Subtree mode', `Subtree with root ${id} set to mode ${mode}`);
+        }
+        break;
+      }
       case TREE_OPERATION_UPDATE_TREE_BASE_DURATION: {
         id = operations[i + 1];
 
@@ -329,8 +338,24 @@ function updateTree(
         i += 3;
         break;
       }
+      case TREE_OPERATION_UPDATE_ERRORS_OR_WARNINGS: {
+        id = operations[i + 1];
+        const numErrors = operations[i + 2];
+        const numWarnings = operations[i + 3];
+
+        i += 4;
+
+        if (__DEBUG__) {
+          debug(
+            'Warnings and Errors update',
+            `fiber ${id} has ${numErrors} errors and ${numWarnings} warnings`,
+          );
+        }
+        break;
+      }
+
       default:
-        throw Error(`Unsupported Bridge operation ${operation}`);
+        throw Error(`Unsupported Bridge operation "${operation}"`);
     }
   }
 
