@@ -7,54 +7,62 @@
  * @flow
  */
 
+import type {CommitDataFrontend} from './types';
+
 import * as React from 'react';
 import {useEffect, useMemo, useRef, useState} from 'react';
 import AutoSizer from 'react-virtualized-auto-sizer';
 import {FixedSizeList} from 'react-window';
 import SnapshotCommitListItem from './SnapshotCommitListItem';
 import {minBarWidth} from './constants';
+import {formatDuration, formatTime} from './utils';
+import Tooltip from './Tooltip';
 
 import styles from './SnapshotCommitList.css';
 
 export type ItemData = {|
-  commitDurations: Array<number>,
   commitTimes: Array<number>,
   filteredCommitIndices: Array<number>,
   maxDuration: number,
   selectedCommitIndex: number | null,
   selectedFilteredCommitIndex: number | null,
   selectCommitIndex: (index: number) => void,
+  setHoveredCommitIndex: (index: number) => void,
   startCommitDrag: (newDragState: DragState) => void,
+  totalDurations: Array<number>,
 |};
 
 type Props = {|
-  commitDurations: Array<number>,
+  commitData: CommitDataFrontend,
   commitTimes: Array<number>,
   filteredCommitIndices: Array<number>,
   selectedCommitIndex: number | null,
   selectedFilteredCommitIndex: number | null,
   selectCommitIndex: (index: number) => void,
+  totalDurations: Array<number>,
 |};
 
 export default function SnapshotCommitList({
-  commitDurations,
+  commitData,
   commitTimes,
   filteredCommitIndices,
   selectedCommitIndex,
   selectedFilteredCommitIndex,
   selectCommitIndex,
+  totalDurations,
 }: Props) {
   return (
     <AutoSizer>
       {({height, width}) => (
         <List
-          commitDurations={commitDurations}
+          commitData={commitData}
           commitTimes={commitTimes}
           height={height}
           filteredCommitIndices={filteredCommitIndices}
           selectedCommitIndex={selectedCommitIndex}
           selectedFilteredCommitIndex={selectedFilteredCommitIndex}
           selectCommitIndex={selectCommitIndex}
+          totalDurations={totalDurations}
           width={width}
         />
       )}
@@ -63,13 +71,14 @@ export default function SnapshotCommitList({
 }
 
 type ListProps = {|
-  commitDurations: Array<number>,
+  commitData: CommitDataFrontend,
   commitTimes: Array<number>,
   height: number,
   filteredCommitIndices: Array<number>,
   selectedCommitIndex: number | null,
   selectedFilteredCommitIndex: number | null,
   selectCommitIndex: (index: number) => void,
+  totalDurations: Array<number>,
   width: number,
 |};
 
@@ -80,13 +89,14 @@ type DragState = {
 };
 
 function List({
-  commitDurations,
+  commitData,
   selectedCommitIndex,
   commitTimes,
   height,
   filteredCommitIndices,
   selectedFilteredCommitIndex,
   selectCommitIndex,
+  totalDurations,
   width,
 }: ListProps) {
   const listRef = useRef<FixedSizeList<ItemData> | null>(null);
@@ -108,8 +118,8 @@ function List({
     [filteredCommitIndices, width],
   );
   const maxDuration = useMemo(
-    () => commitDurations.reduce((max, duration) => Math.max(max, duration), 0),
-    [commitDurations],
+    () => totalDurations.reduce((max, duration) => Math.max(max, duration), 0),
+    [totalDurations],
   );
 
   const maxCommitIndex = filteredCommitIndices.length - 1;
@@ -166,42 +176,126 @@ function List({
     }
   }, [dragState]);
 
+  const [hoveredCommitIndex, setHoveredCommitIndex] = useState<number | null>(
+    null,
+  );
+
   // Pass required contextual data down to the ListItem renderer.
   const itemData = useMemo<ItemData>(
     () => ({
-      commitDurations,
       commitTimes,
       filteredCommitIndices,
       maxDuration,
       selectedCommitIndex,
       selectedFilteredCommitIndex,
       selectCommitIndex,
+      setHoveredCommitIndex,
       startCommitDrag: setDragState,
+      totalDurations,
     }),
     [
-      commitDurations,
       commitTimes,
       filteredCommitIndices,
       maxDuration,
       selectedCommitIndex,
       selectedFilteredCommitIndex,
       selectCommitIndex,
+      setHoveredCommitIndex,
+      totalDurations,
     ],
   );
 
+  let tooltipLabel = null;
+  if (hoveredCommitIndex !== null) {
+    const {
+      duration,
+      effectDuration,
+      passiveEffectDuration,
+      priorityLevel,
+      timestamp,
+    } = commitData[hoveredCommitIndex];
+
+    // Only some React versions include commit durations.
+    // Show a richer tooltip only for builds that have that info.
+    if (
+      effectDuration !== null ||
+      passiveEffectDuration !== null ||
+      priorityLevel !== null
+    ) {
+      tooltipLabel = (
+        <ul className={styles.TooltipList}>
+          {priorityLevel !== null && (
+            <li className={styles.TooltipListItem}>
+              <label className={styles.TooltipLabel}>Priority</label>
+              <span className={styles.TooltipValue}>{priorityLevel}</span>
+            </li>
+          )}
+          <li className={styles.TooltipListItem}>
+            <label className={styles.TooltipLabel}>Committed at</label>
+            <span className={styles.TooltipValue}>
+              {formatTime(timestamp)}s
+            </span>
+          </li>
+          <li className={styles.TooltipListItem}>
+            <div className={styles.DurationsWrapper}>
+              <label className={styles.TooltipLabel}>Durations</label>
+              <ul className={styles.DurationsList}>
+                <li className={styles.DurationsListItem}>
+                  <label className={styles.DurationsLabel}>Render</label>
+                  <span className={styles.DurationsValue}>
+                    {formatDuration(duration)}ms
+                  </span>
+                </li>
+                {effectDuration !== null && (
+                  <li className={styles.DurationsListItem}>
+                    <label className={styles.DurationsLabel}>
+                      Layout effects
+                    </label>
+                    <span className={styles.DurationsValue}>
+                      {formatDuration(effectDuration)}ms
+                    </span>
+                  </li>
+                )}
+                {passiveEffectDuration !== null && (
+                  <li className={styles.DurationsListItem}>
+                    <label className={styles.DurationsLabel}>
+                      Passive effects
+                    </label>
+                    <span className={styles.DurationsValue}>
+                      {formatDuration(passiveEffectDuration)}ms
+                    </span>
+                  </li>
+                )}
+              </ul>
+            </div>
+          </li>
+        </ul>
+      );
+    } else {
+      tooltipLabel = `${formatDuration(duration)}ms at ${formatTime(
+        timestamp,
+      )}s`;
+    }
+  }
+
   return (
-    <div ref={divRef} style={{height, width}}>
-      <FixedSizeList
-        className={styles.List}
-        layout="horizontal"
-        height={height}
-        itemCount={filteredCommitIndices.length}
-        itemData={itemData}
-        itemSize={itemSize}
-        ref={(listRef: any) /* Flow bug? */}
-        width={width}>
-        {SnapshotCommitListItem}
-      </FixedSizeList>
-    </div>
+    <Tooltip className={styles.Tooltip} label={tooltipLabel}>
+      <div
+        ref={divRef}
+        style={{height, width}}
+        onMouseLeave={() => setHoveredCommitIndex(null)}>
+        <FixedSizeList
+          className={styles.List}
+          layout="horizontal"
+          height={height}
+          itemCount={filteredCommitIndices.length}
+          itemData={itemData}
+          itemSize={itemSize}
+          ref={(listRef: any) /* Flow bug? */}
+          width={width}>
+          {SnapshotCommitListItem}
+        </FixedSizeList>
+      </div>
+    </Tooltip>
   );
 }
