@@ -8,6 +8,11 @@
  */
 
 import type {ElementType} from 'react-devtools-shared/src/types';
+import type {SerializedElement} from '../Components/types';
+import type {
+  TimelineData,
+  TimelineDataExport,
+} from 'react-devtools-timeline/src/types';
 
 export type CommitTreeNode = {|
   id: number,
@@ -25,12 +30,6 @@ export type CommitTree = {|
   rootID: number,
 |};
 
-export type Interaction = {|
-  id: number,
-  name: string,
-  timestamp: number,
-|};
-
 export type SnapshotNode = {|
   id: number,
   children: Array<number>,
@@ -46,15 +45,20 @@ export type ChangeDescription = {|
   isFirstMount: boolean,
   props: Array<string> | null,
   state: Array<string> | null,
+  hooks?: Array<number> | null,
 |};
 
 export type CommitDataFrontend = {|
   // Map of Fiber (ID) to a description of what changed in this commit.
   changeDescriptions: Map<number, ChangeDescription> | null,
 
-  // How long was this commit?
+  // How long was the render phase?
   duration: number,
 
+  // How long was the layout commit phase?
+  // Note that not all builds of React expose this property.
+  effectDuration: number | null,
+
   // Map of Fiber (ID) to actual duration for this commit;
   // Fibers that did not render will not have entries in this Map.
   fiberActualDurations: Map<number, number>,
@@ -63,14 +67,18 @@ export type CommitDataFrontend = {|
   // Fibers that did not render will not have entries in this Map.
   fiberSelfDurations: Map<number, number>,
 
-  // Which interactions (IDs) were associated with this commit.
-  interactionIDs: Array<number>,
+  // How long was the passive commit phase?
+  // Note that not all builds of React expose this property.
+  passiveEffectDuration: number | null,
 
   // Priority level of the commit (if React provided this info)
   priorityLevel: string | null,
 
   // When did this commit occur (relative to the start of profiling)
   timestamp: number,
+
+  // Fiber(s) responsible for scheduling this update.
+  updaters: Array<SerializedElement> | null,
 |};
 
 export type ProfilingDataForRootFrontend = {|
@@ -85,12 +93,6 @@ export type ProfilingDataForRootFrontend = {|
   // This info can be used along with commitOperations to reconstruct the tree for any commit.
   initialTreeBaseDurations: Map<number, number>,
 
-  // All interactions recorded (for this root) during the current session.
-  interactionCommits: Map<number, Array<number>>,
-
-  // All interactions recorded (for this root) during the current session.
-  interactions: Map<number, Interaction>,
-
   // List of tree mutation that occur during profiling.
   // These mutations can be used along with initial snapshots to reconstruct the tree for any commit.
   operations: Array<Array<number>>,
@@ -105,21 +107,30 @@ export type ProfilingDataForRootFrontend = {|
 
 // Combination of profiling data collected by the renderer interface (backend) and Store (frontend).
 export type ProfilingDataFrontend = {|
-  // Profiling data per root.
+  // Legacy profiling data is per renderer + root.
   dataForRoots: Map<number, ProfilingDataForRootFrontend>,
+
+  // Timeline data is per rederer.
+  timelineData: Array<TimelineData>,
+
+  // Some functionality should be disabled for imported data.
+  // e.g. DevTools should not try to sync selection between Components and Profiler tabs,
+  // even if there are Fibers with the same IDs.
   imported: boolean,
 |};
 
 export type CommitDataExport = {|
   changeDescriptions: Array<[number, ChangeDescription]> | null,
   duration: number,
+  effectDuration: number | null,
   // Tuple of fiber ID and actual duration
   fiberActualDurations: Array<[number, number]>,
   // Tuple of fiber ID and computed "self" duration
   fiberSelfDurations: Array<[number, number]>,
-  interactionIDs: Array<number>,
+  passiveEffectDuration: number | null,
   priorityLevel: string | null,
   timestamp: number,
+  updaters: Array<SerializedElement> | null,
 |};
 
 export type ProfilingDataForRootExport = {|
@@ -127,9 +138,6 @@ export type ProfilingDataForRootExport = {|
   displayName: string,
   // Tuple of Fiber ID and base duration
   initialTreeBaseDurations: Array<[number, number]>,
-  // Tuple of Interaction ID and commit indices
-  interactionCommits: Array<[number, Array<number>]>,
-  interactions: Array<[number, Interaction]>,
   operations: Array<Array<number>>,
   rootID: number,
   snapshots: Array<[number, SnapshotNode]>,
@@ -137,6 +145,12 @@ export type ProfilingDataForRootExport = {|
 
 // Serializable version of ProfilingDataFrontend data.
 export type ProfilingDataExport = {|
-  version: 4,
+  version: 5,
+
+  // Legacy profiling data is per renderer + root.
   dataForRoots: Array<ProfilingDataForRootExport>,
+
+  // Timeline data is per rederer.
+  // Note that old exported profiles won't contain this key.
+  timelineData?: Array<TimelineDataExport>,
 |};
