@@ -17,6 +17,7 @@ import CustomObject from './CustomObject';
 import EdgeCaseObjects from './EdgeCaseObjects.js';
 import NestedProps from './NestedProps';
 import SimpleValues from './SimpleValues';
+import SymbolKeys from './SymbolKeys';
 
 // TODO Add Immutable JS example
 
@@ -32,6 +33,7 @@ export default function InspectableElements() {
       <CustomObject />
       <EdgeCaseObjects />
       <CircularReferences />
+      <SymbolKeys />
     </Fragment>
   );
 }
