@@ -11,12 +11,6 @@ function ignoreStrings(
   methodName: string,
   stringsToIgnore: Array<string>,
 ): void {
-  // HACKY In the test harness, DevTools overrides the parent window's console.
-  // Our test app code uses the iframe's console though.
-  // To simulate a more accurate end-to-end environment,
-  // the shell's console patching should pass through to the parent override methods.
-  const originalMethod = window.parent.console[methodName];
-
   console[methodName] = (...args) => {
     const maybeString = args[0];
     if (typeof maybeString === 'string') {
@@ -26,7 +20,12 @@ function ignoreStrings(
         }
       }
     }
-    originalMethod(...args);
+
+    // HACKY In the test harness, DevTools overrides the parent window's console.
+    // Our test app code uses the iframe's console though.
+    // To simulate a more accurate end-to-end environment,
+    // the shell's console patching should pass through to the parent override methods.
+    window.parent.console[methodName](...args);
   };
 }
 
@@ -37,3 +36,7 @@ export function ignoreErrors(errorsToIgnore: Array<string>): void {
 export function ignoreWarnings(warningsToIgnore: Array<string>): void {
   ignoreStrings('warn', warningsToIgnore);
 }
+
+export function ignoreLogs(logsToIgnore: Array<string>): void {
+  ignoreStrings('log', logsToIgnore);
+}
