@@ -8,18 +8,15 @@
  */
 
 export {
-  createPortal,
-  unstable_batchedUpdates,
-  flushSync,
   __SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED,
-  version,
+  createPortal,
   createRoot,
-  createRoot as unstable_createRoot,
-  createBlockingRoot,
-  createBlockingRoot as unstable_createBlockingRoot,
-  unstable_flushControlled,
-  unstable_scheduleHydration,
-  unstable_runWithPriority,
+  hydrateRoot,
+  flushSync,
+  unstable_batchedUpdates,
   unstable_createEventHandle,
+  unstable_flushControlled,
   unstable_isNewReconciler,
+  unstable_runWithPriority, // DO NOT USE: Temporarily exposed to migrate off of Scheduler.runWithPriority.
+  version,
 } from './src/client/ReactDOM';
