@@ -15,11 +15,9 @@ let ReactDOMServer;
 let ReactTestUtils;
 
 describe('ReactComponent', () => {
-  function normalizeCodeLocInfo(str) {
-    return str && str.replace(/\(at .+?:\d+\)/g, '(at **)');
-  }
-
   beforeEach(() => {
+    jest.resetModules();
+
     React = require('react');
     ReactDOM = require('react-dom');
     ReactDOMServer = require('react-dom/server');
@@ -465,20 +463,11 @@ describe('ReactComponent', () => {
     };
     const element = <div>{[children]}</div>;
     const container = document.createElement('div');
-    let ex;
-    try {
+    expect(() => {
       ReactDOM.render(element, container);
-    } catch (e) {
-      ex = e;
-    }
-    expect(ex).toBeDefined();
-    expect(normalizeCodeLocInfo(ex.message)).toBe(
-      'Objects are not valid as a React child (found: object with keys {x, y, z}).' +
-        (__DEV__
-          ? ' If you meant to render a collection of children, use ' +
-            'an array instead.' +
-            '\n    in div (at **)'
-          : ''),
+    }).toThrowError(
+      'Objects are not valid as a React child (found: object with keys {x, y, z}). ' +
+        'If you meant to render a collection of children, use an array instead.',
     );
   });
 
@@ -494,21 +483,12 @@ describe('ReactComponent', () => {
       }
     }
     const container = document.createElement('div');
-    let ex;
-    try {
+    expect(() => {
       ReactDOM.render(<Foo />, container);
-    } catch (e) {
-      ex = e;
-    }
-    expect(ex).toBeDefined();
-    expect(normalizeCodeLocInfo(ex.message)).toBe(
+    }).toThrowError(
       'Objects are not valid as a React child (found: object with keys {a, b, c}).' +
-        (__DEV__
-          ? ' If you meant to render a collection of children, use ' +
-            'an array instead.\n' +
-            '    in div (at **)\n' +
-            '    in Foo (at **)'
-          : ''),
+        ' If you meant to render a collection of children, use an array ' +
+        'instead.',
     );
   });
 
@@ -519,20 +499,12 @@ describe('ReactComponent', () => {
       z: <span />,
     };
     const element = <div>{[children]}</div>;
-    let ex;
-    try {
+    expect(() => {
       ReactDOMServer.renderToString(element);
-    } catch (e) {
-      ex = e;
-    }
-    expect(ex).toBeDefined();
-    expect(normalizeCodeLocInfo(ex.message)).toBe(
-      'Objects are not valid as a React child (found: object with keys {x, y, z}).' +
-        (__DEV__
-          ? ' If you meant to render a collection of children, use ' +
-            'an array instead.' +
-            '\n    in div (at **)'
-          : ''),
+    }).toThrowError(
+      'Objects are not valid as a React child (found: object with keys {x, y, z}). ' +
+        'If you meant to render a collection of children, use ' +
+        'an array instead.',
     );
   });
 
@@ -548,21 +520,12 @@ describe('ReactComponent', () => {
       }
     }
     const container = document.createElement('div');
-    let ex;
-    try {
+    expect(() => {
       ReactDOMServer.renderToString(<Foo />, container);
-    } catch (e) {
-      ex = e;
-    }
-    expect(ex).toBeDefined();
-    expect(normalizeCodeLocInfo(ex.message)).toBe(
-      'Objects are not valid as a React child (found: object with keys {a, b, c}).' +
-        (__DEV__
-          ? ' If you meant to render a collection of children, use ' +
-            'an array instead.\n' +
-            '    in div (at **)\n' +
-            '    in Foo (at **)'
-          : ''),
+    }).toThrowError(
+      'Objects are not valid as a React child (found: object with keys {a, b, c}). ' +
+        'If you meant to render a collection of children, use ' +
+        'an array instead.',
     );
   });
 
