@@ -142,7 +142,7 @@ describe('ReactDOMTextarea', () => {
             return value;
           },
           set: function(val) {
-            value = '' + val;
+            value = String(val);
             counter++;
           },
         });
@@ -219,6 +219,36 @@ describe('ReactDOMTextarea', () => {
     expect(node.value).toEqual('foo');
   });
 
+  it('should throw when value is set to a Temporal-like object', () => {
+    class TemporalLike {
+      valueOf() {
+        // Throwing here is the behavior of ECMAScript "Temporal" date/time API.
+        // See https://tc39.es/proposal-temporal/docs/plaindate.html#valueOf
+        throw new TypeError('prod message');
+      }
+      toString() {
+        return '2020-01-01';
+      }
+    }
+    const container = document.createElement('div');
+    const stub = <textarea value="giraffe" onChange={emptyFunction} />;
+    const node = renderTextarea(stub, container);
+
+    expect(node.value).toBe('giraffe');
+
+    const test = () =>
+      ReactDOM.render(
+        <textarea value={new TemporalLike()} onChange={emptyFunction} />,
+        container,
+      );
+    expect(() =>
+      expect(test).toThrowError(new TypeError('prod message')),
+    ).toErrorDev(
+      'Form field values (value, checked, defaultValue, or defaultChecked props) must be ' +
+        'strings, not TemporalLike. This value must be coerced to a string before before using it here.',
+    );
+  });
+
   it('should take updates to `defaultValue` for uncontrolled textarea', () => {
     const container = document.createElement('div');
 
