@@ -668,16 +668,22 @@ describe('ReactLegacyErrorBoundaries', () => {
 
   it('logs a single error using both error boundaries', () => {
     const container = document.createElement('div');
-    expect(() =>
-      ReactDOM.render(
-        <BothErrorBoundaries>
-          <BrokenRender />
-        </BothErrorBoundaries>,
-        container,
-      ),
-    ).toErrorDev('The above error occurred in the <BrokenRender> component', {
-      logAllErrors: true,
-    });
+    spyOnDev(console, 'error');
+    ReactDOM.render(
+      <BothErrorBoundaries>
+        <BrokenRender />
+      </BothErrorBoundaries>,
+      container,
+    );
+    if (__DEV__) {
+      expect(console.error).toHaveBeenCalledTimes(2);
+      expect(console.error.calls.argsFor(0)[0]).toContain(
+        'ReactDOM.render is no longer supported',
+      );
+      expect(console.error.calls.argsFor(1)[0]).toContain(
+        'The above error occurred in the <BrokenRender> component:',
+      );
+    }
 
     expect(container.firstChild.textContent).toBe('Caught an error: Hello.');
     expect(log).toEqual([
