@@ -3,6 +3,8 @@
  *
  * This source code is licensed under the MIT license found in the
  * LICENSE file in the root directory of this source tree.
+ *
+ * @flow
  */
 
 import {canUseDOM} from 'shared/ExecutionEnvironment';
@@ -20,7 +22,7 @@ import {canUseDOM} from 'shared/ExecutionEnvironment';
  * @internal
  * @license Modernizr 3.0.0pre (Custom Build) | MIT
  */
-function isEventSupported(eventNameSuffix) {
+function isEventSupported(eventNameSuffix: string): boolean {
   if (!canUseDOM) {
     return false;
   }
@@ -31,7 +33,7 @@ function isEventSupported(eventNameSuffix) {
   if (!isSupported) {
     const element = document.createElement('div');
     element.setAttribute(eventName, 'return;');
-    isSupported = typeof element[eventName] === 'function';
+    isSupported = typeof (element: any)[eventName] === 'function';
   }
 
   return isSupported;
