@@ -38,4 +38,13 @@ module.exports = {
   get flattenStyle() {
     return require('./flattenStyle');
   },
+  get legacySendAccessibilityEvent() {
+    return require('./legacySendAccessibilityEvent');
+  },
+  get RawEventEmitter() {
+    return require('./RawEventEmitter').default;
+  },
+  get CustomEvent() {
+    return require('./CustomEvent').default;
+  },
 };
