@@ -14,30 +14,33 @@
  * environment.
  */
 
-import type {ReactModelRoot} from 'react-flight/inline-typed';
+import {readModule} from 'react-noop-renderer/flight-modules';
 
-import ReactFlightClient from 'react-flight';
+import ReactFlightClient from 'react-client/flight';
 
 type Source = Array<string>;
 
-const {
-  createResponse,
-  getModelRoot,
-  processStringChunk,
-  complete,
-} = ReactFlightClient({
+const {createResponse, processStringChunk, close} = ReactFlightClient({
   supportsBinaryStreams: false,
+  resolveModuleReference(bundlerConfig: null, idx: string) {
+    return idx;
+  },
+  preloadModule(idx: string) {},
+  requireModule(idx: string) {
+    return readModule(idx);
+  },
+  parseModel(response: Response, json) {
+    return JSON.parse(json, response._fromJSON);
+  },
 });
 
-function read<T>(source: Source): ReactModelRoot<T> {
-  let response = createResponse(source);
+function read<T>(source: Source): T {
+  const response = createResponse(source, null);
   for (let i = 0; i < source.length; i++) {
     processStringChunk(response, source[i], 0);
   }
-  complete(response);
-  return getModelRoot(response);
+  close(response);
+  return response.readRoot();
 }
 
-export default {
-  read,
-};
+export {read};
