@@ -7,8 +7,7 @@
  * @flow
  */
 
-export type RootTag = 0 | 1 | 2;
+export type RootTag = 0 | 1;
 
 export const LegacyRoot = 0;
-export const BlockingRoot = 1;
-export const ConcurrentRoot = 2;
+export const ConcurrentRoot = 1;
