@@ -32,7 +32,8 @@ export type WorkTag =
   | 21
   | 22
   | 23
-  | 24;
+  | 24
+  | 25;
 
 export const FunctionComponent = 0;
 export const ClassComponent = 1;
@@ -54,8 +55,8 @@ export const LazyComponent = 16;
 export const IncompleteClassComponent = 17;
 export const DehydratedFragment = 18;
 export const SuspenseListComponent = 19;
-export const FundamentalComponent = 20;
 export const ScopeComponent = 21;
-export const Block = 22;
-export const OffscreenComponent = 23;
-export const LegacyHiddenComponent = 24;
+export const OffscreenComponent = 22;
+export const LegacyHiddenComponent = 23;
+export const CacheComponent = 24;
+export const TracingMarkerComponent = 25;
