@@ -8,45 +8,47 @@
  */
 
 export {
+  __SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED,
+  act as unstable_act,
   Children,
-  createRef,
   Component,
+  Fragment,
+  Profiler,
   PureComponent,
+  StrictMode,
+  Suspense,
+  SuspenseList,
+  cloneElement,
   createContext,
+  createElement,
+  createFactory,
+  createRef,
+  createServerContext,
   forwardRef,
+  isValidElement,
   lazy,
   memo,
+  startTransition,
+  unstable_Cache,
+  unstable_DebugTracingMode,
+  unstable_Offscreen,
+  unstable_getCacheSignal,
+  unstable_getCacheForType,
+  unstable_useCacheRefresh,
+  useId,
   useCallback,
   useContext,
+  useDebugValue,
+  useDeferredValue,
   useEffect,
   useImperativeHandle,
-  useDebugValue,
+  useInsertionEffect,
   useLayoutEffect,
   useMemo,
   useReducer,
   useRef,
   useState,
-  useMutableSource as unstable_useMutableSource,
-  createMutableSource as unstable_createMutableSource,
-  Fragment,
-  Profiler,
-  StrictMode,
-  Suspense,
-  unstable_LegacyHidden,
-  createElement,
-  cloneElement,
-  isValidElement,
+  useSyncExternalStore,
+  useTransition,
   version,
-  __SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED,
-  createFactory,
-  // exposeConcurrentModeAPIs
-  useTransition as unstable_useTransition,
-  useDeferredValue as unstable_useDeferredValue,
-  startTransition as unstable_startTransition,
-  SuspenseList as unstable_SuspenseList,
-  // enableBlocksAPI
-  block as unstable_block,
-  unstable_useOpaqueIdentifier,
-  // enableDebugTracing
-  unstable_DebugTracingMode,
 } from './src/React';
