@@ -8,32 +8,39 @@
  */
 
 export {
+  __SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED,
+  act as unstable_act,
   Children,
-  createRef,
   Component,
+  Fragment,
+  Profiler,
   PureComponent,
+  StrictMode,
+  Suspense,
+  cloneElement,
   createContext,
+  createElement,
+  createFactory,
+  createRef,
   forwardRef,
+  isValidElement,
   lazy,
   memo,
+  startTransition,
+  useId,
   useCallback,
   useContext,
+  useDebugValue,
+  useDeferredValue,
   useEffect,
   useImperativeHandle,
-  useDebugValue,
+  useInsertionEffect,
   useLayoutEffect,
   useMemo,
   useReducer,
   useRef,
   useState,
-  Fragment,
-  Profiler,
-  StrictMode,
-  Suspense,
-  createElement,
-  cloneElement,
-  isValidElement,
+  useSyncExternalStore,
+  useTransition,
   version,
-  __SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED,
-  createFactory,
 } from './src/React';
