@@ -16,7 +16,10 @@ import {
   REACT_SUSPENSE_TYPE,
   REACT_SUSPENSE_LIST_TYPE,
   REACT_LEGACY_HIDDEN_TYPE,
+  REACT_OFFSCREEN_TYPE,
   REACT_SCOPE_TYPE,
+  REACT_CACHE_TYPE,
+  REACT_TRACING_MARKER_TYPE,
 } from 'shared/ReactSymbols';
 
 import {Component, PureComponent} from './ReactBaseClasses';
@@ -32,32 +35,37 @@ import {createContext} from './ReactContext';
 import {lazy} from './ReactLazy';
 import {forwardRef} from './ReactForwardRef';
 import {memo} from './ReactMemo';
-import {block} from './ReactBlock';
 import {
+  getCacheSignal,
+  getCacheForType,
   useCallback,
   useContext,
   useEffect,
   useImperativeHandle,
   useDebugValue,
+  useInsertionEffect,
   useLayoutEffect,
   useMemo,
   useMutableSource,
+  useSyncExternalStore,
   useReducer,
   useRef,
   useState,
   useTransition,
   useDeferredValue,
-  useOpaqueIdentifier,
+  useId,
+  useCacheRefresh,
 } from './ReactHooks';
 import {
   createElementWithValidation,
   createFactoryWithValidation,
   cloneElementWithValidation,
 } from './ReactElementValidator';
+import {createServerContext} from './ReactServerContext';
 import {createMutableSource} from './ReactMutableSource';
 import ReactSharedInternals from './ReactSharedInternals';
-import {createFundamental} from './ReactFundamental';
 import {startTransition} from './ReactStartTransition';
+import {act} from './ReactAct';
 
 // TODO: Move this branching into the other module instead and just re-export.
 const createElement = __DEV__ ? createElementWithValidation : createElementProd;
@@ -79,6 +87,7 @@ export {
   Component,
   PureComponent,
   createContext,
+  createServerContext,
   forwardRef,
   lazy,
   memo,
@@ -87,9 +96,11 @@ export {
   useEffect,
   useImperativeHandle,
   useDebugValue,
+  useInsertionEffect,
   useLayoutEffect,
   useMemo,
   useMutableSource,
+  useSyncExternalStore,
   useReducer,
   useRef,
   useState,
@@ -111,11 +122,15 @@ export {
   useDeferredValue,
   REACT_SUSPENSE_LIST_TYPE as SuspenseList,
   REACT_LEGACY_HIDDEN_TYPE as unstable_LegacyHidden,
-  // enableBlocksAPI
-  block,
-  // enableFundamentalAPI
-  createFundamental as unstable_createFundamental,
+  REACT_OFFSCREEN_TYPE as unstable_Offscreen,
+  getCacheSignal as unstable_getCacheSignal,
+  getCacheForType as unstable_getCacheForType,
+  useCacheRefresh as unstable_useCacheRefresh,
+  REACT_CACHE_TYPE as unstable_Cache,
   // enableScopeAPI
   REACT_SCOPE_TYPE as unstable_Scope,
-  useOpaqueIdentifier as unstable_useOpaqueIdentifier,
+  // enableTransitionTracing
+  REACT_TRACING_MARKER_TYPE as unstable_TracingMarker,
+  useId,
+  act,
 };
