@@ -9,6 +9,8 @@
 
 'use strict';
 
+import {enableSymbolFallbackForWWW} from 'shared/ReactFeatureFlags';
+
 let React;
 let ReactDOM;
 let ReactTestUtils;
@@ -20,10 +22,12 @@ describe('ReactElement', () => {
   beforeEach(() => {
     jest.resetModules();
 
-    // Delete the native Symbol if we have one to ensure we test the
-    // unpolyfilled environment.
-    originalSymbol = global.Symbol;
-    global.Symbol = undefined;
+    if (enableSymbolFallbackForWWW) {
+      // Delete the native Symbol if we have one to ensure we test the
+      // unpolyfilled environment.
+      originalSymbol = global.Symbol;
+      global.Symbol = undefined;
+    }
 
     React = require('react');
     ReactDOM = require('react-dom');
@@ -38,9 +42,12 @@ describe('ReactElement', () => {
   });
 
   afterEach(() => {
-    global.Symbol = originalSymbol;
+    if (enableSymbolFallbackForWWW) {
+      global.Symbol = originalSymbol;
+    }
   });
 
+  // @gate enableSymbolFallbackForWWW
   it('uses the fallback value when in an environment without Symbol', () => {
     expect((<div />).$$typeof).toBe(0xeac7);
   });
@@ -296,6 +303,7 @@ describe('ReactElement', () => {
 
   // NOTE: We're explicitly not using JSX here. This is intended to test
   // classic JS without JSX.
+  // @gate enableSymbolFallbackForWWW
   it('identifies valid elements', () => {
     class Component extends React.Component {
       render() {
@@ -446,7 +454,43 @@ describe('ReactElement', () => {
 
   // NOTE: We're explicitly not using JSX here. This is intended to test
   // classic JS without JSX.
+  // @gate !enableSymbolFallbackForWWW
   it('identifies elements, but not JSON, if Symbols are supported', () => {
+    class Component extends React.Component {
+      render() {
+        return React.createElement('div');
+      }
+    }
+
+    expect(React.isValidElement(React.createElement('div'))).toEqual(true);
+    expect(React.isValidElement(React.createElement(Component))).toEqual(true);
+
+    expect(React.isValidElement(null)).toEqual(false);
+    expect(React.isValidElement(true)).toEqual(false);
+    expect(React.isValidElement({})).toEqual(false);
+    expect(React.isValidElement('string')).toEqual(false);
+    if (!__EXPERIMENTAL__) {
+      let factory;
+      expect(() => {
+        factory = React.createFactory('div');
+      }).toWarnDev(
+        'Warning: React.createFactory() is deprecated and will be removed in a ' +
+          'future major release. Consider using JSX or use React.createElement() ' +
+          'directly instead.',
+        {withoutStack: true},
+      );
+      expect(React.isValidElement(factory)).toEqual(false);
+    }
+    expect(React.isValidElement(Component)).toEqual(false);
+    expect(React.isValidElement({type: 'div', props: {}})).toEqual(false);
+
+    const jsonElement = JSON.stringify(React.createElement('div'));
+    expect(React.isValidElement(JSON.parse(jsonElement))).toBe(false);
+  });
+
+  // NOTE: We're explicitly not using JSX here. This is intended to test
+  // classic JS without JSX.
+  it('identifies elements, but not JSON, if Symbols are supported (with polyfill)', () => {
     // Rudimentary polyfill
     // Once all jest engines support Symbols natively we can swap this to test
     // WITH native Symbols by default.
