@@ -5,22 +5,19 @@
  * LICENSE file in the root directory of this source tree.
  */
 
-import assign from 'object-assign';
 import * as Scheduler from 'scheduler';
-import * as SchedulerTracing from 'scheduler/tracing';
 import ReactCurrentDispatcher from '../ReactCurrentDispatcher';
+import ReactCurrentActQueue from '../ReactCurrentActQueue';
 import ReactCurrentOwner from '../ReactCurrentOwner';
 import ReactDebugCurrentFrame from '../ReactDebugCurrentFrame';
-import IsSomeRendererActing from '../IsSomeRendererActing';
 import ReactCurrentBatchConfig from '../ReactCurrentBatchConfig';
+import {enableServerContext} from 'shared/ReactFeatureFlags';
+import {ContextRegistry} from '../ReactServerContextRegistry';
 
 const ReactSharedInternals = {
   ReactCurrentDispatcher,
   ReactCurrentOwner,
-  IsSomeRendererActing,
   ReactCurrentBatchConfig,
-  // Used by renderers to avoid bundling object-assign twice in UMD bundles:
-  assign,
 
   // Re-export the schedule API(s) for UMD bundles.
   // This avoids introducing a dependency on a new UMD global in a minor update,
@@ -28,11 +25,15 @@ const ReactSharedInternals = {
   // This re-export is only required for UMD bundles;
   // CJS bundles use the shared NPM package.
   Scheduler,
-  SchedulerTracing,
 };
 
 if (__DEV__) {
+  ReactSharedInternals.ReactCurrentActQueue = ReactCurrentActQueue;
   ReactSharedInternals.ReactDebugCurrentFrame = ReactDebugCurrentFrame;
 }
 
+if (enableServerContext) {
+  ReactSharedInternals.ContextRegistry = ContextRegistry;
+}
+
 export default ReactSharedInternals;
