@@ -7,4 +7,4 @@
 
 'use strict';
 
-export * from './src/Scheduler';
+export * from './src/forks/Scheduler';
