@@ -8,4 +8,8 @@
 
 export const enableSchedulerDebugging = false;
 export const enableIsInputPending = false;
-export const enableProfiling = __PROFILE__;
+export const enableProfiling = false;
+export const enableIsInputPendingContinuous = false;
+export const frameYieldMs = 5;
+export const continuousYieldMs = 50;
+export const maxYieldMs = 300;
