@@ -13,7 +13,6 @@
 'use strict';
 
 let Scheduler;
-let sharedProfilingArray;
 // let runWithPriority;
 let ImmediatePriority;
 let UserBlockingPriority;
@@ -44,7 +43,8 @@ function priorityLevelToString(priorityLevel) {
 }
 
 describe('Scheduler', () => {
-  if (!__PROFILE__) {
+  const {enableProfiling} = require('scheduler/src/SchedulerFeatureFlags');
+  if (!enableProfiling) {
     // The tests in this suite only apply when profiling is on
     it('profiling APIs are not available', () => {
       Scheduler = require('scheduler');
@@ -58,10 +58,6 @@ describe('Scheduler', () => {
     jest.mock('scheduler', () => require('scheduler/unstable_mock'));
     Scheduler = require('scheduler');
 
-    sharedProfilingArray = new Int32Array(
-      Scheduler.unstable_Profiling.sharedProfilingBuffer,
-    );
-
     // runWithPriority = Scheduler.unstable_runWithPriority;
     ImmediatePriority = Scheduler.unstable_ImmediatePriority;
     UserBlockingPriority = Scheduler.unstable_UserBlockingPriority;
@@ -75,20 +71,6 @@ describe('Scheduler', () => {
     // shouldYield = Scheduler.unstable_shouldYield;
   });
 
-  const PRIORITY = 0;
-  const CURRENT_TASK_ID = 1;
-  const CURRENT_RUN_ID = 2;
-  const QUEUE_SIZE = 3;
-
-  afterEach(() => {
-    if (sharedProfilingArray[QUEUE_SIZE] !== 0) {
-      throw Error(
-        'Test exited, but the shared profiling buffer indicates that a task ' +
-          'is still running',
-      );
-    }
-  });
-
   const TaskStartEvent = 1;
   const TaskCompleteEvent = 2;
   const TaskErrorEvent = 3;
@@ -271,23 +253,6 @@ describe('Scheduler', () => {
     return '\n' + result;
   }
 
-  function getProfilingInfo() {
-    const queueSize = sharedProfilingArray[QUEUE_SIZE];
-    if (queueSize === 0) {
-      return 'Empty Queue';
-    }
-    const priorityLevel = sharedProfilingArray[PRIORITY];
-    if (priorityLevel === 0) {
-      return 'Suspended, Queue Size: ' + queueSize;
-    }
-    return (
-      `Task: ${sharedProfilingArray[CURRENT_TASK_ID]}, ` +
-      `Run: ${sharedProfilingArray[CURRENT_RUN_ID]}, ` +
-      `Priority: ${priorityLevelToString(priorityLevel)}, ` +
-      `Queue Size: ${sharedProfilingArray[QUEUE_SIZE]}`
-    );
-  }
-
   it('creates a basic flamegraph', () => {
     Scheduler.unstable_Profiling.startLoggingProfilingEvents();
 
@@ -296,35 +261,27 @@ describe('Scheduler', () => {
       NormalPriority,
       () => {
         Scheduler.unstable_advanceTime(300);
-        Scheduler.unstable_yieldValue(getProfilingInfo());
+        Scheduler.unstable_yieldValue('Yield 1');
         scheduleCallback(
           UserBlockingPriority,
           () => {
-            Scheduler.unstable_yieldValue(getProfilingInfo());
+            Scheduler.unstable_yieldValue('Yield 2');
             Scheduler.unstable_advanceTime(300);
           },
           {label: 'Bar'},
         );
         Scheduler.unstable_advanceTime(100);
-        Scheduler.unstable_yieldValue('Yield');
+        Scheduler.unstable_yieldValue('Yield 3');
         return () => {
-          Scheduler.unstable_yieldValue(getProfilingInfo());
+          Scheduler.unstable_yieldValue('Yield 4');
           Scheduler.unstable_advanceTime(300);
         };
       },
       {label: 'Foo'},
     );
-    expect(Scheduler).toFlushAndYieldThrough([
-      'Task: 1, Run: 1, Priority: Normal, Queue Size: 1',
-      'Yield',
-    ]);
+    expect(Scheduler).toFlushAndYieldThrough(['Yield 1', 'Yield 3']);
     Scheduler.unstable_advanceTime(100);
-    expect(Scheduler).toFlushAndYield([
-      'Task: 2, Run: 2, Priority: User-blocking, Queue Size: 2',
-      'Task: 1, Run: 3, Priority: Normal, Queue Size: 1',
-    ]);
-
-    expect(getProfilingInfo()).toEqual('Empty Queue');
+    expect(Scheduler).toFlushAndYield(['Yield 2', 'Yield 4']);
 
     expect(stopProfilingAndPrintFlamegraph()).toEqual(
       `
@@ -339,19 +296,16 @@ Task 1 [Normal]              │  ████████░░░░░░░
     Scheduler.unstable_Profiling.startLoggingProfilingEvents();
 
     const task = scheduleCallback(NormalPriority, () => {
-      Scheduler.unstable_yieldValue(getProfilingInfo());
+      Scheduler.unstable_yieldValue('Yield 1');
       Scheduler.unstable_advanceTime(300);
-      Scheduler.unstable_yieldValue('Yield');
+      Scheduler.unstable_yieldValue('Yield 2');
       return () => {
         Scheduler.unstable_yieldValue('Continuation');
         Scheduler.unstable_advanceTime(200);
       };
     });
 
-    expect(Scheduler).toFlushAndYieldThrough([
-      'Task: 1, Run: 1, Priority: Normal, Queue Size: 1',
-      'Yield',
-    ]);
+    expect(Scheduler).toFlushAndYieldThrough(['Yield 1', 'Yield 2']);
     Scheduler.unstable_advanceTime(100);
 
     cancelCallback(task);
@@ -391,28 +345,25 @@ Task 1 [Normal]              │██████🡐 errored
     Scheduler.unstable_Profiling.startLoggingProfilingEvents();
 
     const task1 = scheduleCallback(NormalPriority, () => {
-      Scheduler.unstable_yieldValue(getProfilingInfo());
+      Scheduler.unstable_yieldValue('Yield 1');
       Scheduler.unstable_advanceTime(300);
-      Scheduler.unstable_yieldValue('Yield');
+      Scheduler.unstable_yieldValue('Yield 2');
       return () => {
         Scheduler.unstable_yieldValue('Continuation');
         Scheduler.unstable_advanceTime(200);
       };
     });
     const task2 = scheduleCallback(NormalPriority, () => {
-      Scheduler.unstable_yieldValue(getProfilingInfo());
+      Scheduler.unstable_yieldValue('Yield 3');
       Scheduler.unstable_advanceTime(300);
-      Scheduler.unstable_yieldValue('Yield');
+      Scheduler.unstable_yieldValue('Yield 4');
       return () => {
         Scheduler.unstable_yieldValue('Continuation');
         Scheduler.unstable_advanceTime(200);
       };
     });
 
-    expect(Scheduler).toFlushAndYieldThrough([
-      'Task: 1, Run: 1, Priority: Normal, Queue Size: 2',
-      'Yield',
-    ]);
+    expect(Scheduler).toFlushAndYieldThrough(['Yield 1', 'Yield 2']);
     Scheduler.unstable_advanceTime(100);
 
     cancelCallback(task1);
