@@ -49,7 +49,6 @@ export type ReactConsumer<T> = {
   ref: null,
   props: {
     children: (value: T) => ReactNodeList,
-    unstable_observedBits?: number,
     ...
   },
   ...
@@ -59,7 +58,6 @@ export type ReactContext<T> = {
   $$typeof: Symbol | number,
   Consumer: ReactContext<T>,
   Provider: ReactProviderType<T>,
-  _calculateChangedBits: ((a: T, b: T) => number) | null,
   _currentValue: T,
   _currentValue2: T,
   _threadCount: number,
@@ -69,9 +67,23 @@ export type ReactContext<T> = {
   // This value may be added by application code
   // to improve DEV tooling display names
   displayName?: string,
+
+  // only used by ServerContext
+  _defaultValue: T,
+  _globalName: string,
   ...
 };
 
+export type ServerContextJSONValue =
+  | string
+  | boolean
+  | number
+  | null
+  | $ReadOnlyArray<ServerContextJSONValue>
+  | {+[key: string]: ServerContextJSONValue};
+
+export type ReactServerContext<T: any> = ReactContext<T>;
+
 export type ReactPortal = {
   $$typeof: Symbol | number,
   key: null | string,
@@ -86,58 +98,6 @@ export type RefObject = {|
   current: any,
 |};
 
-export type EventPriority = 0 | 1 | 2;
-
-export const DiscreteEvent: EventPriority = 0;
-export const UserBlockingEvent: EventPriority = 1;
-export const ContinuousEvent: EventPriority = 2;
-
-export type ReactFundamentalComponentInstance<C, H> = {|
-  currentFiber: Object,
-  instance: mixed,
-  prevProps: null | Object,
-  props: Object,
-  impl: ReactFundamentalImpl<C, H>,
-  state: Object,
-|};
-
-export type ReactFundamentalImpl<C, H> = {
-  displayName: string,
-  reconcileChildren: boolean,
-  getInitialState?: (props: Object) => Object,
-  getInstance: (context: C, props: Object, state: Object) => H,
-  getServerSideString?: (context: C, props: Object) => string,
-  getServerSideStringClose?: (context: C, props: Object) => string,
-  onMount: (context: C, instance: mixed, props: Object, state: Object) => void,
-  shouldUpdate?: (
-    context: C,
-    prevProps: null | Object,
-    nextProps: Object,
-    state: Object,
-  ) => boolean,
-  onUpdate?: (
-    context: C,
-    instance: mixed,
-    prevProps: null | Object,
-    nextProps: Object,
-    state: Object,
-  ) => void,
-  onUnmount?: (
-    context: C,
-    instance: mixed,
-    props: Object,
-    state: Object,
-  ) => void,
-  onHydrate?: (context: C, props: Object, state: Object) => boolean,
-  onFocus?: (context: C, props: Object, state: Object) => boolean,
-  ...
-};
-
-export type ReactFundamentalComponent<C, H> = {|
-  $$typeof: Symbol | number,
-  impl: ReactFundamentalImpl<C, H>,
-|};
-
 export type ReactScope = {|
   $$typeof: Symbol | number,
 |};
@@ -197,14 +157,18 @@ export type MutableSource<Source: $NonMaybeType<mixed>> = {|
   // Used to detect multiple renderers using the same mutable source.
   _currentPrimaryRenderer?: Object | null,
   _currentSecondaryRenderer?: Object | null,
+
+  // DEV only
+  // Used to detect side effects that update a mutable source during render.
+  // See https://github.com/facebook/react/issues/19948
+  _currentlyRenderingFiber?: Fiber | null,
+  _initialVersionAsOfFirstRender?: MutableSourceVersion | null,
 |};
 
 // The subset of a Thenable required by things thrown by Suspense.
 // This doesn't require a value to be passed to either handler.
 export interface Wakeable {
   then(onFulfill: () => mixed, onReject: () => mixed): void | Wakeable;
-  // Special flag to opt out of tracing interactions across a Suspense boundary.
-  __reactDoNotTraceInteractions?: boolean;
 }
 
 // The subset of a Promise that React APIs rely on. This resolves a value.
@@ -216,3 +180,12 @@ export interface Thenable<+R> {
     onReject: (error: mixed) => void | Thenable<U> | U,
   ): void | Thenable<U>;
 }
+
+export type OffscreenMode =
+  | 'hidden'
+  | 'unstable-defer-without-hiding'
+  | 'visible';
+
+export type StartTransitionOptions = {
+  name?: string,
+};
