@@ -1,6 +1,7 @@
 'use strict';
 
 module.exports = {
+  globalSetup: require.resolve('./setupGlobal.js'),
   haste: {
     hasteImplModulePath: require.resolve('./noHaste.js'),
   },
@@ -24,6 +25,5 @@ module.exports = {
 
   testSequencer: require.resolve('./jestSequencer'),
 
-  // TODO: Upgrade to Jest 26 which uses jsdom 16 by default.
-  testEnvironment: require.resolve('jest-environment-jsdom-sixteen'),
+  testEnvironment: 'jsdom',
 };
