@@ -13,6 +13,16 @@ module.exports = function shouldIgnoreConsoleError(format, args) {
         // Ignore it too.
         return true;
       }
+      if (
+        format.indexOf('ReactDOM.render is no longer supported in React 18') !==
+          -1 ||
+        format.indexOf(
+          'ReactDOM.hydrate is no longer supported in React 18'
+        ) !== -1
+      ) {
+        // We haven't finished migrating our tests to use createRoot.
+        return true;
+      }
     }
   } else {
     if (
@@ -25,16 +35,6 @@ module.exports = function shouldIgnoreConsoleError(format, args) {
       // They are noisy too so we'll try to ignore them.
       return true;
     }
-    if (
-      format.indexOf(
-        'act(...) is not supported in production builds of React'
-      ) === 0
-    ) {
-      // We don't yet support act() for prod builds, and warn for it.
-      // But we'd like to use act() ourselves for prod builds.
-      // Let's ignore the warning and #yolo.
-      return true;
-    }
   }
   // Looks legit
   return false;
