@@ -6,7 +6,7 @@ const clear = require('clear');
 const {join, relative} = require('path');
 const theme = require('../theme');
 
-module.exports = ({build}) => {
+module.exports = async ({build}) => {
   const commandPath = relative(
     process.env.PWD,
     join(__dirname, '../download-experimental-build.js')
